"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var index_exports = {};
__export(index_exports, {
  useBattery: () => useBattery,
  useClickOutside: () => useClickOutside,
  useClipboard: () => useClipboard,
  useDebounce: () => useDebounce,
  useGeolocation: () => useGeolocation,
  useHasRendered: () => useHasRendered,
  useHover: () => useHover,
  useIdle: () => useIdle,
  useIntersectionObserver: () => useIntersectionObserver,
  useIsBrowser: () => useIsBrowser,
  useIsServer: () => useIsServer,
  useLocalStorage: () => useLocalStorage
});
module.exports = __toCommonJS(index_exports);

// src/hooks/useIsBrowser.ts
var import_react = require("react");
function useIsBrowser() {
  const [isBrowser, setIsBrowser] = (0, import_react.useState)(false);
  (0, import_react.useEffect)(() => {
    setIsBrowser(true);
  }, []);
  return isBrowser;
}

// src/hooks/useIsServer.ts
var import_react2 = require("react");
function useIsServer() {
  const [isServer, setIsServer] = (0, import_react2.useState)(true);
  (0, import_react2.useEffect)(() => {
    setIsServer(false);
  }, []);
  return isServer;
}

// src/hooks/useBattery.ts
var import_react3 = require("react");
function useBattery() {
  const [state, setState] = (0, import_react3.useState)({
    isSupported: false,
    isLoading: true,
    error: null,
    battery: null
  });
  (0, import_react3.useEffect)(() => {
    if (!navigator || !("getBattery" in navigator)) {
      setState({
        isSupported: false,
        isLoading: false,
        error: null,
        battery: null
      });
      return;
    }
    let batteryManager = null;
    let isComponentMounted = true;
    const updateBatteryState = (manager) => {
      if (!isComponentMounted) return;
      if (!manager) {
        setState((prev) => ({
          ...prev,
          isLoading: false,
          error: new Error("Could not access battery information"),
          battery: null
        }));
        return;
      }
      setState({
        isSupported: true,
        isLoading: false,
        error: null,
        battery: {
          charging: manager.charging,
          chargingTime: manager.chargingTime,
          dischargingTime: manager.dischargingTime,
          level: manager.level,
          connected: manager.charging || manager.level === 1 && manager.chargingTime === 0
        }
      });
    };
    const setupBatteryListeners = (manager) => {
      if (!manager) return;
      manager.addEventListener(
        "chargingchange",
        () => updateBatteryState(manager)
      );
      manager.addEventListener(
        "chargingtimechange",
        () => updateBatteryState(manager)
      );
      manager.addEventListener(
        "dischargingtimechange",
        () => updateBatteryState(manager)
      );
      manager.addEventListener(
        "levelchange",
        () => updateBatteryState(manager)
      );
      batteryManager = manager;
    };
    const getBatteryInfo = async () => {
      try {
        const manager = await navigator.getBattery();
        if (isComponentMounted) {
          updateBatteryState(manager);
          setupBatteryListeners(manager);
        }
      } catch (error) {
        if (isComponentMounted) {
          setState({
            isSupported: true,
            // API exists but there was an error
            isLoading: false,
            error: error instanceof Error ? error : new Error("Unknown battery error"),
            battery: null
          });
        }
      }
    };
    getBatteryInfo();
    return () => {
      isComponentMounted = false;
      if (batteryManager) {
        batteryManager.removeEventListener(
          "chargingchange",
          updateBatteryState
        );
        batteryManager.removeEventListener(
          "chargingtimechange",
          updateBatteryState
        );
        batteryManager.removeEventListener(
          "dischargingtimechange",
          updateBatteryState
        );
        batteryManager.removeEventListener("levelchange", updateBatteryState);
      }
    };
  }, []);
  return state;
}

// src/hooks/useClickOutside.ts
var import_react4 = require("react");
var import_react5 = require("react");
function useClickOutside(onClickOutside, options = {}) {
  const {
    enabled = true,
    ignoreRefs = [],
    mouseEvents = ["mousedown"],
    listenForTouchEvents = true,
    ignoreScrollbar = true
  } = options;
  const ref = (0, import_react4.useRef)(null);
  const handleClickOutside = (0, import_react4.useCallback)(
    (event) => {
      if (!enabled) return;
      const target = event.target;
      if (ignoreScrollbar && event instanceof MouseEvent) {
        if (event.clientX >= document.documentElement.clientWidth) {
          return;
        }
      }
      if (ref.current && ref.current.contains(target)) {
        return;
      }
      for (const ignoreRef of ignoreRefs) {
        if (ignoreRef.current && ignoreRef.current.contains(target)) {
          return;
        }
      }
      onClickOutside(event);
    },
    [enabled, ignoreRefs, ignoreScrollbar, onClickOutside]
  );
  (0, import_react4.useEffect)(() => {
    if (!enabled) return;
    mouseEvents.forEach((mouseEvent) => {
      document.addEventListener(
        mouseEvent,
        handleClickOutside
      );
    });
    if (listenForTouchEvents) {
      document.addEventListener(
        "touchstart",
        handleClickOutside
      );
    }
    return () => {
      mouseEvents.forEach((mouseEvent) => {
        document.removeEventListener(
          mouseEvent,
          handleClickOutside
        );
      });
      if (listenForTouchEvents) {
        document.removeEventListener(
          "touchstart",
          handleClickOutside
        );
      }
    };
  }, [enabled, mouseEvents, listenForTouchEvents, handleClickOutside]);
  return ref;
}

// src/hooks/useClipboard.ts
var import_react6 = require("react");
function useClipboard(options = {}) {
  const {
    resetDelay = 2e3,
    autoReset = true,
    initialValue = "",
    modernOnly = false,
    onCopySuccess,
    onCopyError
  } = options;
  const isBrowser = useIsBrowser();
  const [value, setValue] = (0, import_react6.useState)(initialValue);
  const [status, setStatus] = (0, import_react6.useState)("idle");
  const [error, setError] = (0, import_react6.useState)(null);
  const isModernSupported = isBrowser && typeof navigator !== "undefined" && navigator.clipboard && typeof navigator.clipboard.writeText === "function";
  const isLegacySupported = isBrowser && typeof document !== "undefined" && typeof document.execCommand === "function";
  const isSupported = isModernSupported || !modernOnly && isLegacySupported;
  (0, import_react6.useEffect)(() => {
    if (status !== "idle" && autoReset) {
      const timeoutId = setTimeout(() => {
        setStatus("idle");
      }, resetDelay);
      return () => clearTimeout(timeoutId);
    }
  }, [status, resetDelay, autoReset]);
  const copyWithLegacyMethod = (0, import_react6.useCallback)((text) => {
    try {
      const textarea = document.createElement("textarea");
      textarea.value = text;
      textarea.style.position = "absolute";
      textarea.style.opacity = "0";
      textarea.style.pointerEvents = "none";
      textarea.style.zIndex = "-1";
      document.body.appendChild(textarea);
      textarea.focus();
      textarea.select();
      const success = document.execCommand("copy");
      document.body.removeChild(textarea);
      return success;
    } catch (err) {
      return false;
    }
  }, []);
  const copyToClipboard = (0, import_react6.useCallback)(
    async (text) => {
      if (!isBrowser) {
        setStatus("error");
        const browserError = new Error(
          "Cannot copy to clipboard on the server"
        );
        setError(browserError);
        onCopyError?.(browserError);
        return false;
      }
      const valueToCopy = text !== void 0 ? text : value;
      setValue(valueToCopy);
      try {
        let success = false;
        if (isModernSupported) {
          await navigator.clipboard.writeText(valueToCopy);
          success = true;
        } else if (!modernOnly && isLegacySupported) {
          success = copyWithLegacyMethod(valueToCopy);
        } else {
          throw new Error("Clipboard API not supported in this environment");
        }
        if (success) {
          setStatus("copied");
          setError(null);
          onCopySuccess?.(valueToCopy);
          return true;
        } else {
          throw new Error("Failed to copy to clipboard");
        }
      } catch (err) {
        setStatus("error");
        const copyError = err instanceof Error ? err : new Error(String(err));
        setError(copyError);
        onCopyError?.(copyError);
        return false;
      }
    },
    [
      isBrowser,
      value,
      isModernSupported,
      modernOnly,
      isLegacySupported,
      copyWithLegacyMethod,
      onCopySuccess,
      onCopyError
    ]
  );
  const reset = (0, import_react6.useCallback)(() => {
    setStatus("idle");
    setError(null);
  }, []);
  return {
    copyToClipboard,
    value,
    status,
    isSuccess: status === "copied",
    isError: status === "error",
    reset,
    error,
    isSupported
  };
}

// src/hooks/useDebounce.ts
var import_react7 = require("react");
function useDebounce(fn, options = {}) {
  const {
    delay = 500,
    leading = false,
    maxWait,
    trackPending = false
  } = options;
  const timeoutRef = (0, import_react7.useRef)(null);
  const lastCalledRef = (0, import_react7.useRef)(0);
  const maxTimeoutRef = (0, import_react7.useRef)(null);
  const lastArgsRef = (0, import_react7.useRef)(null);
  const lastResultRef = (0, import_react7.useRef)(void 0);
  const [isPending, setIsPending] = (0, import_react7.useState)(false);
  const cancel = (0, import_react7.useCallback)(() => {
    if (timeoutRef.current) {
      clearTimeout(timeoutRef.current);
      timeoutRef.current = null;
    }
    if (maxTimeoutRef.current) {
      clearTimeout(maxTimeoutRef.current);
      maxTimeoutRef.current = null;
    }
    if (trackPending && isPending) {
      setIsPending(false);
    }
  }, [isPending, trackPending]);
  const executeFunction = (0, import_react7.useCallback)(
    (...args) => {
      lastCalledRef.current = Date.now();
      lastArgsRef.current = null;
      if (trackPending) {
        setIsPending(false);
      }
      try {
        const result = fn(...args);
        lastResultRef.current = result;
        return result;
      } catch (error) {
        lastResultRef.current = void 0;
        throw error;
      }
    },
    [fn, trackPending]
  );
  const flush = (0, import_react7.useCallback)(() => {
    if (timeoutRef.current || maxTimeoutRef.current) {
      if (lastArgsRef.current) {
        const result = executeFunction(...lastArgsRef.current);
        lastArgsRef.current = null;
        return result;
      }
    }
    return lastResultRef.current;
  }, [executeFunction]);
  (0, import_react7.useEffect)(() => {
    return () => cancel();
  }, [cancel]);
  const debounced = (0, import_react7.useCallback)(
    (...args) => {
      lastArgsRef.current = args;
      const now = Date.now();
      const timeSinceLastCall = now - lastCalledRef.current;
      if (trackPending && !isPending) {
        setIsPending(true);
      }
      if (timeoutRef.current) {
        clearTimeout(timeoutRef.current);
      }
      if (leading && (lastCalledRef.current === 0 || timeSinceLastCall >= delay)) {
        executeFunction(...args);
      } else {
        timeoutRef.current = setTimeout(() => {
          if (maxTimeoutRef.current) {
            clearTimeout(maxTimeoutRef.current);
            maxTimeoutRef.current = null;
          }
          executeFunction(...args);
        }, delay);
        if (maxWait && !maxTimeoutRef.current && timeSinceLastCall >= maxWait) {
          maxTimeoutRef.current = setTimeout(() => {
            if (timeoutRef.current) {
              clearTimeout(timeoutRef.current);
              timeoutRef.current = null;
            }
            executeFunction(...args);
          }, maxWait - timeSinceLastCall);
        }
      }
    },
    [delay, executeFunction, isPending, leading, maxWait, trackPending]
  );
  return [debounced, cancel, isPending, flush];
}

// src/hooks/useGeolocation.ts
var import_react8 = require("react");
function useGeolocation(options = {}) {
  const {
    enableHighAccuracy = false,
    timeout = 1e4,
    maximumAge = 0,
    watchPosition: shouldWatch = false,
    autoRequest = true,
    onSuccess,
    onError,
    onPermissionChange
  } = options;
  const isBrowser = useIsBrowser();
  const [position, setPosition] = (0, import_react8.useState)(null);
  const [error, setError] = (0, import_react8.useState)(
    null
  );
  const [isLoading, setIsLoading] = (0, import_react8.useState)(false);
  const [isWatching, setIsWatching] = (0, import_react8.useState)(false);
  const [permissionState, setPermissionState] = (0, import_react8.useState)("unknown");
  const watchIdRef = (0, import_react8.useRef)(null);
  const isSupported = isBrowser && typeof navigator !== "undefined" && "geolocation" in navigator;
  const formatPosition = (0, import_react8.useCallback)(
    (position2) => {
      return {
        latitude: position2.latitude,
        longitude: position2.longitude,
        accuracy: position2.accuracy,
        altitude: position2.altitude,
        altitudeAccuracy: position2.altitudeAccuracy,
        heading: position2.heading,
        speed: position2.speed,
        timestamp: position2.timestamp
      };
    },
    []
  );
  const handlePositionSuccess = (0, import_react8.useCallback)(
    (rawPosition) => {
      const formattedPosition = formatPosition(rawPosition);
      setPosition(formattedPosition);
      setError(null);
      setIsLoading(false);
      onSuccess?.(formattedPosition);
    },
    [formatPosition, onSuccess]
  );
  const handlePositionError = (0, import_react8.useCallback)(
    (positionError) => {
      setError(positionError);
      setIsLoading(false);
      onError?.(positionError);
      if (positionError instanceof GeolocationPositionError && positionError.code === 1 /* PERMISSION_DENIED */) {
        setPermissionState("denied");
        onPermissionChange?.("denied");
      }
    },
    [onError, onPermissionChange]
  );
  const getPosition = (0, import_react8.useCallback)(() => {
    return new Promise((resolve, reject) => {
      if (!isSupported) {
        const error2 = new Error(
          "Geolocation is not supported by this browser."
        );
        setError(error2);
        reject(error2);
        return;
      }
      setIsLoading(true);
      setError(null);
      navigator.geolocation.getCurrentPosition(
        (rawPosition) => {
          const formattedPosition = {
            latitude: rawPosition.coords.latitude,
            longitude: rawPosition.coords.longitude,
            accuracy: rawPosition.coords.accuracy,
            altitude: rawPosition.coords.altitude,
            altitudeAccuracy: rawPosition.coords.altitudeAccuracy,
            heading: rawPosition.coords.heading,
            speed: rawPosition.coords.speed,
            timestamp: rawPosition.timestamp
          };
          handlePositionSuccess(formattedPosition);
          resolve(formattedPosition);
        },
        (positionError) => {
          handlePositionError(positionError);
          reject(positionError);
        },
        {
          enableHighAccuracy,
          timeout,
          maximumAge
        }
      );
    });
  }, [
    isSupported,
    enableHighAccuracy,
    timeout,
    maximumAge,
    handlePositionSuccess,
    handlePositionError
  ]);
  const watchPosition = (0, import_react8.useCallback)(() => {
    if (!isSupported) {
      const error2 = new Error("Geolocation is not supported by this browser.");
      setError(error2);
      return;
    }
    if (watchIdRef.current !== null) {
      navigator.geolocation.clearWatch(watchIdRef.current);
    }
    setIsLoading(true);
    setError(null);
    setIsWatching(true);
    watchIdRef.current = navigator.geolocation.watchPosition(
      (rawPosition) => {
        const formattedPosition = {
          latitude: rawPosition.coords.latitude,
          longitude: rawPosition.coords.longitude,
          accuracy: rawPosition.coords.accuracy,
          altitude: rawPosition.coords.altitude,
          altitudeAccuracy: rawPosition.coords.altitudeAccuracy,
          heading: rawPosition.coords.heading,
          speed: rawPosition.coords.speed,
          timestamp: rawPosition.timestamp
        };
        handlePositionSuccess(formattedPosition);
      },
      handlePositionError,
      {
        enableHighAccuracy,
        timeout,
        maximumAge
      }
    );
  }, [
    isSupported,
    enableHighAccuracy,
    timeout,
    maximumAge,
    handlePositionSuccess,
    handlePositionError
  ]);
  const stopWatching = (0, import_react8.useCallback)(() => {
    if (!isSupported || watchIdRef.current === null) return;
    navigator.geolocation.clearWatch(watchIdRef.current);
    watchIdRef.current = null;
    setIsWatching(false);
  }, [isSupported]);
  const checkPermissionState = (0, import_react8.useCallback)(async () => {
    if (!isBrowser) return "unknown";
    if (navigator.permissions && navigator.permissions.query) {
      try {
        const permissionStatus = await navigator.permissions.query({
          name: "geolocation"
        });
        const state = permissionStatus.state;
        setPermissionState(state);
        onPermissionChange?.(state);
        permissionStatus.addEventListener("change", () => {
          const newState = permissionStatus.state;
          setPermissionState(newState);
          onPermissionChange?.(newState);
        });
        return state;
      } catch (err) {
        console.warn("Permissions API error:", err);
      }
    }
    if (position) {
      setPermissionState("granted");
      return "granted";
    } else if (error instanceof GeolocationPositionError && error.code === 1 /* PERMISSION_DENIED */) {
      setPermissionState("denied");
      return "denied";
    }
    return "prompt";
  }, [isBrowser, position, error, onPermissionChange]);
  (0, import_react8.useEffect)(() => {
    if (isSupported) {
      checkPermissionState();
    }
  }, [isSupported, checkPermissionState]);
  (0, import_react8.useEffect)(() => {
    if (!isSupported || !autoRequest) return;
    if (shouldWatch) {
      watchPosition();
    } else {
      getPosition().catch(() => {
      });
    }
    return () => {
      if (watchIdRef.current !== null) {
        navigator.geolocation.clearWatch(watchIdRef.current);
        watchIdRef.current = null;
        setIsWatching(false);
      }
    };
  }, [isSupported, autoRequest, shouldWatch, watchPosition, getPosition]);
  return {
    position,
    error,
    isLoading,
    isWatching,
    permissionState,
    getPosition,
    watchPosition,
    stopWatching,
    isSupported
  };
}

// src/hooks/useHover.ts
var import_react9 = require("react");
function useHover(options = {}) {
  const {
    enterDelay = 0,
    leaveDelay = 0,
    enabled: initialEnabled = true,
    supportTouch = false,
    onHoverStart,
    onHoverEnd,
    onEnabledChange
  } = options;
  const isBrowser = useIsBrowser();
  const [isHovered, setIsHovered] = (0, import_react9.useState)(false);
  const [isEnabled, setIsEnabled] = (0, import_react9.useState)(initialEnabled);
  const hoverRef = (0, import_react9.useRef)(null);
  const enterTimeoutRef = (0, import_react9.useRef)(null);
  const leaveTimeoutRef = (0, import_react9.useRef)(null);
  const isTouchDevice = (0, import_react9.useRef)(false);
  const clearTimeouts = (0, import_react9.useCallback)(() => {
    if (enterTimeoutRef.current) {
      clearTimeout(enterTimeoutRef.current);
      enterTimeoutRef.current = null;
    }
    if (leaveTimeoutRef.current) {
      clearTimeout(leaveTimeoutRef.current);
      leaveTimeoutRef.current = null;
    }
  }, []);
  const handleEnter = (0, import_react9.useCallback)(
    (event) => {
      if (event.type.startsWith("mouse") && isTouchDevice.current) {
        return;
      }
      if (event.type.startsWith("touch")) {
        isTouchDevice.current = true;
      }
      if (leaveTimeoutRef.current) {
        clearTimeout(leaveTimeoutRef.current);
        leaveTimeoutRef.current = null;
      }
      if (enterDelay > 0) {
        enterTimeoutRef.current = setTimeout(() => {
          setIsHovered(true);
          onHoverStart?.(event);
        }, enterDelay);
      } else {
        setIsHovered(true);
        onHoverStart?.(event);
      }
    },
    [enterDelay, onHoverStart]
  );
  const handleLeave = (0, import_react9.useCallback)(
    (event) => {
      if (event.type.startsWith("mouse") && isTouchDevice.current) {
        return;
      }
      if (enterTimeoutRef.current) {
        clearTimeout(enterTimeoutRef.current);
        enterTimeoutRef.current = null;
      }
      if (leaveDelay > 0) {
        leaveTimeoutRef.current = setTimeout(() => {
          setIsHovered(false);
          onHoverEnd?.(event);
        }, leaveDelay);
      } else {
        setIsHovered(false);
        onHoverEnd?.(event);
      }
    },
    [leaveDelay, onHoverEnd]
  );
  const enable = (0, import_react9.useCallback)(() => {
    if (!isEnabled) {
      setIsEnabled(true);
      onEnabledChange?.(true);
    }
  }, [isEnabled, onEnabledChange]);
  const disable = (0, import_react9.useCallback)(() => {
    if (isEnabled) {
      setIsEnabled(false);
      setIsHovered(false);
      clearTimeouts();
      onEnabledChange?.(false);
    }
  }, [isEnabled, clearTimeouts, onEnabledChange]);
  (0, import_react9.useEffect)(() => {
    const element = hoverRef.current;
    if (!isBrowser || !element || !isEnabled) {
      return;
    }
    if (typeof window !== "undefined") {
      isTouchDevice.current = "ontouchstart" in window || navigator.maxTouchPoints > 0 || navigator.msMaxTouchPoints > 0;
    }
    element.addEventListener("mouseenter", handleEnter);
    element.addEventListener("mouseleave", handleLeave);
    if (supportTouch) {
      element.addEventListener("touchstart", handleEnter, {
        passive: true
      });
      element.addEventListener("touchend", handleLeave, {
        passive: true
      });
      const handleTouchMove = (event) => {
        const touch = event.touches[0];
        if (touch && element) {
          const rect = element.getBoundingClientRect();
          if (touch.clientX < rect.left || touch.clientX > rect.right || touch.clientY < rect.top || touch.clientY > rect.bottom) {
            handleLeave(event);
          }
        }
      };
      document.addEventListener("touchmove", handleTouchMove, {
        passive: true
      });
      return () => {
        element.removeEventListener("mouseenter", handleEnter);
        element.removeEventListener("mouseleave", handleLeave);
        element.removeEventListener("touchstart", handleEnter);
        element.removeEventListener("touchend", handleLeave);
        document.removeEventListener(
          "touchmove",
          handleTouchMove
        );
        clearTimeouts();
      };
    }
    return () => {
      element.removeEventListener("mouseenter", handleEnter);
      element.removeEventListener("mouseleave", handleLeave);
      clearTimeouts();
    };
  }, [
    isBrowser,
    isEnabled,
    supportTouch,
    handleEnter,
    handleLeave,
    clearTimeouts
  ]);
  (0, import_react9.useEffect)(() => {
    if (initialEnabled !== isEnabled) {
      setIsEnabled(initialEnabled);
    }
  }, [initialEnabled, isEnabled]);
  (0, import_react9.useEffect)(() => {
    return () => {
      clearTimeouts();
    };
  }, [clearTimeouts]);
  return {
    hoverRef,
    isHovered,
    enable,
    disable,
    isEnabled
  };
}

// src/hooks/useIdle.ts
var import_react10 = require("react");
function useIdle(options = {}) {
  const {
    idleTime = 6e4,
    // Default 1 minute
    initialState = false,
    events = [
      "mousemove",
      "mousedown",
      "resize",
      "keydown",
      "touchstart",
      "wheel"
    ],
    minimumMovement = 10,
    trackInBackground = true,
    onIdle,
    onActive,
    enabled: initialEnabled = true,
    idleOnVisibilityHidden = false,
    syncWithStorage = false,
    storageKey = "user-idle-state",
    emitEvents = false,
    eventName = "user-idle-state-change",
    pollingInterval = 1e3
  } = options;
  const isBrowser = useIsBrowser();
  const [isIdle, setIsIdle] = (0, import_react10.useState)(initialState);
  const [isEnabled, setIsEnabled] = (0, import_react10.useState)(initialEnabled);
  const [idleTimeTracked, setIdleTimeTracked] = (0, import_react10.useState)(0);
  const [remainingTimeValue, setRemainingTimeValue] = (0, import_react10.useState)(idleTime);
  const lastActiveRef = (0, import_react10.useRef)(Date.now());
  const timerRef = (0, import_react10.useRef)(null);
  const pollingIntervalRef = (0, import_react10.useRef)(null);
  const lastMousePositionRef = (0, import_react10.useRef)({ x: 0, y: 0 });
  const lastEventHandlerRef = (0, import_react10.useRef)(() => {
  });
  const initTimer = (0, import_react10.useCallback)(() => {
    if (!isBrowser || !isEnabled) return;
    if (timerRef.current) {
      clearTimeout(timerRef.current);
    }
    timerRef.current = setTimeout(() => {
      if (!document.hidden || trackInBackground) {
        setIsIdle(true);
        onIdle?.();
        if (syncWithStorage) {
          try {
            localStorage.setItem(
              storageKey,
              JSON.stringify({
                isIdle: true,
                timestamp: Date.now()
              })
            );
          } catch (e) {
            console.warn("Failed to sync idle state to localStorage", e);
          }
        }
        if (emitEvents && typeof window !== "undefined") {
          const idleEvent = new CustomEvent(eventName, {
            detail: {
              isIdle: true,
              idleTime: Date.now() - lastActiveRef.current,
              timestamp: Date.now()
            }
          });
          window.dispatchEvent(idleEvent);
        }
      }
    }, idleTime);
    return () => {
      if (timerRef.current) {
        clearTimeout(timerRef.current);
      }
    };
  }, [
    isBrowser,
    isEnabled,
    idleTime,
    trackInBackground,
    onIdle,
    syncWithStorage,
    storageKey,
    emitEvents,
    eventName
  ]);
  const startPolling = (0, import_react10.useCallback)(() => {
    if (!isBrowser || !isEnabled) return;
    if (pollingIntervalRef.current) {
      clearInterval(pollingIntervalRef.current);
    }
    pollingIntervalRef.current = setInterval(() => {
      const now = Date.now();
      const elapsed = now - lastActiveRef.current;
      const remaining = Math.max(0, idleTime - elapsed);
      setIdleTimeTracked(elapsed);
      setRemainingTimeValue(remaining);
    }, pollingInterval);
    return () => {
      if (pollingIntervalRef.current) {
        clearInterval(pollingIntervalRef.current);
        pollingIntervalRef.current = null;
      }
    };
  }, [isBrowser, isEnabled, idleTime, pollingInterval]);
  const handleActivity = (0, import_react10.useCallback)(
    (event) => {
      if (!isEnabled) return;
      let shouldReset = true;
      if (event && event.type === "mousemove" && minimumMovement > 0) {
        const mouseEvent = event;
        const { x, y } = lastMousePositionRef.current;
        const deltaX = Math.abs(mouseEvent.clientX - x);
        const deltaY = Math.abs(mouseEvent.clientY - y);
        const distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
        shouldReset = distance >= minimumMovement;
        lastMousePositionRef.current = {
          x: mouseEvent.clientX,
          y: mouseEvent.clientY
        };
      }
      if (shouldReset) {
        const wasIdle = isIdle;
        const now = Date.now();
        lastActiveRef.current = now;
        if (wasIdle) {
          setIsIdle(false);
          onActive?.();
          if (syncWithStorage) {
            try {
              localStorage.setItem(
                storageKey,
                JSON.stringify({
                  isIdle: false,
                  timestamp: now
                })
              );
            } catch (e) {
              console.warn("Failed to sync idle state to localStorage", e);
            }
          }
          if (emitEvents && typeof window !== "undefined") {
            const activeEvent = new CustomEvent(eventName, {
              detail: {
                isIdle: false,
                idleTime: 0,
                timestamp: now
              }
            });
            window.dispatchEvent(activeEvent);
          }
        }
        initTimer();
      }
    },
    [
      isEnabled,
      isIdle,
      minimumMovement,
      onActive,
      initTimer,
      syncWithStorage,
      storageKey,
      emitEvents,
      eventName
    ]
  );
  const handleVisibilityChange = (0, import_react10.useCallback)(() => {
    if (!isEnabled) return;
    if (document.hidden && idleOnVisibilityHidden) {
      setIsIdle(true);
      onIdle?.();
      if (syncWithStorage) {
        try {
          localStorage.setItem(
            storageKey,
            JSON.stringify({
              isIdle: true,
              timestamp: Date.now()
            })
          );
        } catch (e) {
          console.warn("Failed to sync idle state to localStorage", e);
        }
      }
    } else if (!document.hidden && isIdle) {
      handleActivity();
    }
  }, [
    isEnabled,
    idleOnVisibilityHidden,
    isIdle,
    onIdle,
    syncWithStorage,
    storageKey,
    handleActivity
  ]);
  const handleStorageChange = (0, import_react10.useCallback)(
    (e) => {
      if (!isEnabled || !syncWithStorage || e.key !== storageKey) return;
      try {
        const data = e.newValue ? JSON.parse(e.newValue) : null;
        if (data && typeof data.isIdle === "boolean") {
          setIsIdle(data.isIdle);
          if (data.isIdle) {
            onIdle?.();
          } else {
            lastActiveRef.current = data.timestamp || Date.now();
            onActive?.();
            initTimer();
          }
        }
      } catch (err) {
        console.warn("Failed to parse idle state from localStorage", err);
      }
    },
    [isEnabled, syncWithStorage, storageKey, onIdle, onActive, initTimer]
  );
  const setIdleManually = (0, import_react10.useCallback)(() => {
    setIsIdle(true);
    onIdle?.();
    if (syncWithStorage) {
      try {
        localStorage.setItem(
          storageKey,
          JSON.stringify({
            isIdle: true,
            timestamp: Date.now()
          })
        );
      } catch (e) {
        console.warn("Failed to sync idle state to localStorage", e);
      }
    }
    if (emitEvents && typeof window !== "undefined") {
      const idleEvent = new CustomEvent(eventName, {
        detail: {
          isIdle: true,
          idleTime: Date.now() - lastActiveRef.current,
          timestamp: Date.now()
        }
      });
      window.dispatchEvent(idleEvent);
    }
  }, [onIdle, syncWithStorage, storageKey, emitEvents, eventName]);
  const setActiveManually = (0, import_react10.useCallback)(() => {
    lastActiveRef.current = Date.now();
    setIsIdle(false);
    onActive?.();
    initTimer();
    if (syncWithStorage) {
      try {
        localStorage.setItem(
          storageKey,
          JSON.stringify({
            isIdle: false,
            timestamp: Date.now()
          })
        );
      } catch (e) {
        console.warn("Failed to sync idle state to localStorage", e);
      }
    }
    if (emitEvents && typeof window !== "undefined") {
      const activeEvent = new CustomEvent(eventName, {
        detail: {
          isIdle: false,
          idleTime: 0,
          timestamp: Date.now()
        }
      });
      window.dispatchEvent(activeEvent);
    }
  }, [onActive, initTimer, syncWithStorage, storageKey, emitEvents, eventName]);
  const reset = (0, import_react10.useCallback)(() => {
    if (isIdle) {
      setActiveManually();
    } else {
      lastActiveRef.current = Date.now();
      initTimer();
    }
  }, [isIdle, setActiveManually, initTimer]);
  const enable = (0, import_react10.useCallback)(() => {
    if (!isEnabled) {
      setIsEnabled(true);
      lastActiveRef.current = Date.now();
      initTimer();
      startPolling();
    }
  }, [isEnabled, initTimer, startPolling]);
  const disable = (0, import_react10.useCallback)(() => {
    if (isEnabled) {
      setIsEnabled(false);
      if (timerRef.current) {
        clearTimeout(timerRef.current);
        timerRef.current = null;
      }
      if (pollingIntervalRef.current) {
        clearInterval(pollingIntervalRef.current);
        pollingIntervalRef.current = null;
      }
      setIsIdle(false);
    }
  }, [isEnabled]);
  (0, import_react10.useEffect)(() => {
    if (!isBrowser) return;
    if (syncWithStorage) {
      try {
        const storedState = localStorage.getItem(storageKey);
        if (storedState) {
          const { isIdle: storedIsIdle, timestamp } = JSON.parse(storedState);
          if (typeof storedIsIdle === "boolean") {
            setIsIdle(storedIsIdle);
            if (!storedIsIdle && timestamp) {
              lastActiveRef.current = timestamp;
            }
          }
        }
      } catch (e) {
        console.warn("Failed to load idle state from localStorage", e);
      }
    }
    lastEventHandlerRef.current = handleActivity;
    if (isEnabled) {
      lastActiveRef.current = Date.now();
      initTimer();
      startPolling();
    }
    return () => {
      if (timerRef.current) {
        clearTimeout(timerRef.current);
      }
      if (pollingIntervalRef.current) {
        clearInterval(pollingIntervalRef.current);
      }
    };
  }, [
    isBrowser,
    isEnabled,
    initTimer,
    startPolling,
    syncWithStorage,
    storageKey,
    handleActivity
  ]);
  (0, import_react10.useEffect)(() => {
    if (!isBrowser || !isEnabled) return;
    const eventHandler = (event) => {
      lastEventHandlerRef.current(event);
    };
    events.forEach((eventName2) => {
      if (eventName2 === "visibilitychange") {
        document.addEventListener(
          eventName2,
          handleVisibilityChange
        );
      } else {
        window.addEventListener(eventName2, eventHandler, {
          passive: true
        });
      }
    });
    if (syncWithStorage) {
      window.addEventListener("storage", handleStorageChange);
    }
    return () => {
      events.forEach((eventName2) => {
        if (eventName2 === "visibilitychange") {
          document.removeEventListener(
            eventName2,
            handleVisibilityChange
          );
        } else {
          window.removeEventListener(eventName2, eventHandler);
        }
      });
      if (syncWithStorage) {
        window.removeEventListener("storage", handleStorageChange);
      }
    };
  }, [
    isBrowser,
    isEnabled,
    events,
    handleVisibilityChange,
    syncWithStorage,
    handleStorageChange
  ]);
  (0, import_react10.useEffect)(() => {
    if (initialEnabled !== isEnabled) {
      if (initialEnabled) {
        enable();
      } else {
        disable();
      }
    }
  }, [initialEnabled, isEnabled, enable, disable]);
  const idlePercentage = Math.min(
    100,
    Math.floor(idleTimeTracked / idleTime * 100)
  );
  return {
    isIdle,
    idleTime: idleTimeTracked,
    remainingTime: remainingTimeValue,
    lastActive: lastActiveRef.current,
    idlePercentage,
    setIdle: setIdleManually,
    setActive: setActiveManually,
    reset,
    enable,
    disable,
    isEnabled
  };
}

// src/hooks/useIntersectionObserver.ts
var import_react11 = require("react");
function useIntersectionObserver(options = {}) {
  const {
    root = null,
    rootMargin = "0px",
    threshold = 0,
    onEnter,
    onExit,
    triggerOnce = false,
    skip = false,
    initialInView = false,
    delay = 0,
    trackVisibilityChanges = false,
    historySize = 10,
    useIsIntersecting = true
  } = options;
  const isBrowser = useIsBrowser();
  const [isInView, setIsInView] = (0, import_react11.useState)(initialInView);
  const [intersectionRatio, setIntersectionRatio] = (0, import_react11.useState)(0);
  const [entry, setEntry] = (0, import_react11.useState)(null);
  const [entryHistory, setEntryHistory] = (0, import_react11.useState)(
    []
  );
  const [isConnected, setIsConnected] = (0, import_react11.useState)(false);
  const [enteredAt, setEnteredAt] = (0, import_react11.useState)(null);
  const [exitedAt, setExitedAt] = (0, import_react11.useState)(null);
  const [inViewDuration, setInViewDuration] = (0, import_react11.useState)(0);
  const ref = (0, import_react11.useRef)(null);
  const observerRef = (0, import_react11.useRef)(null);
  const skipRef = (0, import_react11.useRef)(skip);
  const wasInViewRef = (0, import_react11.useRef)(initialInView);
  const timerId = (0, import_react11.useRef)(null);
  const durationTimerId = (0, import_react11.useRef)(null);
  const disconnect = (0, import_react11.useCallback)(() => {
    if (observerRef.current) {
      observerRef.current.disconnect();
      setIsConnected(false);
      if (durationTimerId.current !== null) {
        cancelAnimationFrame(durationTimerId.current);
        durationTimerId.current = null;
      }
    }
  }, []);
  const updateInViewDuration = (0, import_react11.useCallback)(() => {
    if (isInView && enteredAt !== null) {
      setInViewDuration(Date.now() - enteredAt);
      durationTimerId.current = requestAnimationFrame(updateInViewDuration);
    } else {
      setInViewDuration(0);
    }
  }, [isInView, enteredAt]);
  const handleIntersection = (0, import_react11.useCallback)(
    (entries) => {
      if (skipRef.current) return;
      const entry2 = entries[0];
      const inThreshold = useIsIntersecting ? entry2.isIntersecting : entry2.intersectionRatio > 0;
      setEntry(entry2);
      setIntersectionRatio(entry2.intersectionRatio);
      if (trackVisibilityChanges) {
        setEntryHistory((prevEntries) => {
          const newEntries = [entry2, ...prevEntries];
          return newEntries.slice(0, historySize);
        });
      }
      if (inThreshold && !wasInViewRef.current) {
        wasInViewRef.current = true;
        setIsInView(true);
        setEnteredAt(Date.now());
        if (durationTimerId.current === null) {
          durationTimerId.current = requestAnimationFrame(updateInViewDuration);
        }
        onEnter?.(entry2);
        if (triggerOnce) {
          disconnect();
        }
      }
      if (!inThreshold && wasInViewRef.current) {
        wasInViewRef.current = false;
        setIsInView(false);
        setExitedAt(Date.now());
        if (durationTimerId.current !== null) {
          cancelAnimationFrame(durationTimerId.current);
          durationTimerId.current = null;
          setInViewDuration(0);
        }
        onExit?.(entry2);
      }
    },
    [
      disconnect,
      onEnter,
      onExit,
      triggerOnce,
      trackVisibilityChanges,
      historySize,
      useIsIntersecting,
      updateInViewDuration
    ]
  );
  const observe = (0, import_react11.useCallback)(() => {
    if (!isBrowser || !ref.current || skipRef.current) return;
    observerRef.current = new IntersectionObserver(handleIntersection, {
      root,
      rootMargin,
      threshold
    });
    observerRef.current.observe(ref.current);
    setIsConnected(true);
    return () => disconnect();
  }, [isBrowser, root, rootMargin, threshold, handleIntersection, disconnect]);
  const reset = (0, import_react11.useCallback)(() => {
    wasInViewRef.current = initialInView;
    setIsInView(initialInView);
    setIntersectionRatio(0);
    setEntry(null);
    setEntryHistory([]);
    setEnteredAt(null);
    setExitedAt(null);
    setInViewDuration(0);
    if (durationTimerId.current !== null) {
      cancelAnimationFrame(durationTimerId.current);
      durationTimerId.current = null;
    }
    disconnect();
    observe();
  }, [initialInView, disconnect, observe]);
  (0, import_react11.useEffect)(() => {
    skipRef.current = skip;
    if (skip && observerRef.current) {
      disconnect();
    } else if (!skip && ref.current && !observerRef.current) {
      observe();
    }
  }, [skip, disconnect, observe]);
  (0, import_react11.useEffect)(() => {
    if (delay > 0) {
      timerId.current = setTimeout(observe, delay);
      return () => {
        if (timerId.current !== null) {
          clearTimeout(timerId.current);
        }
        disconnect();
      };
    } else {
      observe();
      return () => disconnect();
    }
  }, [delay, observe, disconnect]);
  return {
    ref,
    isInView,
    intersectionRatio,
    entry,
    entryHistory,
    reset,
    disconnect,
    observe,
    isConnected,
    enteredAt,
    exitedAt,
    inViewDuration
  };
}

// src/hooks/useHasRendered.ts
var import_react12 = require("react");
function useHasRendered() {
  const hasRenderedRef = (0, import_react12.useRef)(false);
  if (!hasRenderedRef.current) {
    hasRenderedRef.current = true;
    return false;
  }
  return true;
}

// src/hooks/useLocalStorage.ts
var import_react13 = require("react");
function useLocalStorage(key, options = {}) {
  const {
    defaultValue,
    serialize = JSON.stringify,
    deserialize = JSON.parse,
    syncAcrossTabs = true,
    listenToStorageChanges = true,
    onError,
    storageType = "localStorage",
    disabled = false,
    processBeforeStorage = (value) => value,
    processAfterRetrieval = (value) => value
  } = options;
  const isBrowser = useIsBrowser();
  const [storedValue, setStoredValue] = (0, import_react13.useState)(() => {
    if (!isBrowser || disabled) {
      return defaultValue;
    }
    try {
      const storage = window[storageType];
      const item = storage.getItem(key);
      if (item === null) {
        return defaultValue;
      }
      const parsedValue = deserialize(item);
      return processAfterRetrieval(parsedValue);
    } catch (error2) {
      const err = error2 instanceof Error ? error2 : new Error(String(error2));
      if (onError) {
        onError(err, "get");
      } else {
        console.error(`Error reading localStorage key "${key}":`, err);
      }
      return defaultValue;
    }
  });
  const [hasError, setHasError] = (0, import_react13.useState)(false);
  const [error, setError] = (0, import_react13.useState)(null);
  const [exists, setExists] = (0, import_react13.useState)(false);
  const latestValueRef = (0, import_react13.useRef)(storedValue);
  const checkExists = (0, import_react13.useCallback)(() => {
    if (!isBrowser || disabled) {
      setExists(false);
      return false;
    }
    try {
      const storage = window[storageType];
      const exists2 = storage.getItem(key) !== null;
      setExists(exists2);
      return exists2;
    } catch (error2) {
      setExists(false);
      return false;
    }
  }, [isBrowser, key, storageType, disabled]);
  (0, import_react13.useEffect)(() => {
    latestValueRef.current = storedValue;
  }, [storedValue]);
  (0, import_react13.useEffect)(() => {
    if (!isBrowser || !syncAcrossTabs || !listenToStorageChanges || disabled) {
      return;
    }
    const handleStorageChange = (e) => {
      if (e.key !== key || e.storageArea !== window[storageType]) {
        return;
      }
      try {
        if (e.newValue === null) {
          setStoredValue(defaultValue);
          setExists(false);
          return;
        }
        const newValue = deserialize(e.newValue);
        const processedValue = processAfterRetrieval(newValue);
        if (JSON.stringify(processedValue) !== JSON.stringify(latestValueRef.current)) {
          setStoredValue(processedValue);
          setExists(true);
        }
      } catch (error2) {
        const err = error2 instanceof Error ? error2 : new Error(String(error2));
        setHasError(true);
        setError(err);
        if (onError) {
          onError(err, "get");
        } else {
          console.error(`Error syncing localStorage key "${key}":`, err);
        }
      }
    };
    window.addEventListener("storage", handleStorageChange);
    return () => {
      window.removeEventListener("storage", handleStorageChange);
    };
  }, [
    isBrowser,
    key,
    syncAcrossTabs,
    listenToStorageChanges,
    deserialize,
    defaultValue,
    onError,
    storageType,
    disabled,
    processAfterRetrieval
  ]);
  (0, import_react13.useEffect)(() => {
    checkExists();
  }, [checkExists]);
  const setValue = (0, import_react13.useCallback)(
    (valueOrFn) => {
      if (!isBrowser || disabled) {
        const newValue = typeof valueOrFn === "function" ? valueOrFn(latestValueRef.current) : valueOrFn;
        setStoredValue(newValue);
        return;
      }
      try {
        const valueToStore = typeof valueOrFn === "function" ? valueOrFn(latestValueRef.current) : valueOrFn;
        const processedValue = processBeforeStorage(valueToStore);
        setStoredValue(valueToStore);
        setHasError(false);
        setError(null);
        setExists(true);
        const storage = window[storageType];
        storage.setItem(key, serialize(processedValue));
      } catch (error2) {
        const err = error2 instanceof Error ? error2 : new Error(String(error2));
        setHasError(true);
        setError(err);
        if (onError) {
          onError(err, "set");
        } else {
          console.error(`Error setting localStorage key "${key}":`, err);
        }
      }
    },
    [
      isBrowser,
      key,
      serialize,
      onError,
      storageType,
      disabled,
      processBeforeStorage
    ]
  );
  const removeValue = (0, import_react13.useCallback)(() => {
    if (!isBrowser || disabled) {
      setStoredValue(defaultValue);
      return;
    }
    try {
      const storage = window[storageType];
      storage.removeItem(key);
      setStoredValue(defaultValue);
      setHasError(false);
      setError(null);
      setExists(false);
    } catch (error2) {
      const err = error2 instanceof Error ? error2 : new Error(String(error2));
      setHasError(true);
      setError(err);
      if (onError) {
        onError(err, "remove");
      } else {
        console.error(`Error removing localStorage key "${key}":`, err);
      }
    }
  }, [isBrowser, key, defaultValue, onError, storageType, disabled]);
  const resetToDefault = (0, import_react13.useCallback)(() => {
    if (defaultValue === void 0) {
      removeValue();
    } else {
      setValue(defaultValue);
    }
  }, [defaultValue, removeValue, setValue]);
  return {
    value: storedValue,
    setValue,
    removeValue,
    hasError,
    error,
    resetToDefault,
    key,
    exists
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  useBattery,
  useClickOutside,
  useClipboard,
  useDebounce,
  useGeolocation,
  useHasRendered,
  useHover,
  useIdle,
  useIntersectionObserver,
  useIsBrowser,
  useIsServer,
  useLocalStorage
});
